#!/usr/bin/env python
__version__ = '$Revision: 8175 $'.split()[1]
__date__ = '$Date: 2008-01-09 18:14:06 -0500 (Wed, 09 Jan 2008) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
AIS database utilities.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2008
@var __date__: Date of last svn commit
@status: under development
@since: 2008 Jan 09
@license: GPL v2
@undocumented: __version__ __author__ __doc__ parser

@todo: probably lots that can be added here
@todo: createtables for the binary messages
'''

import os
import sys
import ais

dbTypes=(
    'postgres' # AKA postgis
    ,'sqlite' # Only sqlite3.  Time to ditch sqlite2
)
'''
The choices of databases that are supported.
'''

def stdCmdlineOptions(parser,dbType='postgres',verbose=False):
    '''
    Standard command line options
    @param parser: OptionParser parser that will get the additional options
    @param dbType: 'postgres' or 'sqlite'
    '''
    if dbType != 'all' and dbType not in dbTypes:
        print  dbType != 'all'
        print dbTypes not in dbTypes, dbTypes
        # FIX: throw a database type exception
        sys.exit('unknown database type: '+dbType)

    if dbType in ('all','postgres'):
        if verbose: sys.stderr.write('Adding postgres options\n')
        parser.add_option('-d','--database-name',dest='databaseName',default='ais'
                          ,help='Name of database within the postgres server [default: %default]')
        parser.add_option('-D','--database-host',dest='databaseHost',default='localhost'
                          ,help='Host name of the computer serving the dbx [default: %default]')
        #defaultUser = os.genenv('USER')
        defaultUser = os.getlogin()
        parser.add_option('-u','--database-user',dest='databaseUser',default=defaultUser
                          ,help='Host name of the to access the database with [default: %default]')

        parser.add_option('-p','--database-passwd',dest='databasePasswd',default=None
                          ,help='Password to access the database with [default: None]')

    if dbType in ('all','sqlite'):
        if verbose: sys.stderr.write('Adding sqlite options\n')
        parser.add_option('-f','--database-file',dest='databaseFilename',default='ais.db3'
			  ,help='Name of the sqlite3 database file to write [default: %default]')


def createTables(cx,dbType='sqlite',includeList=None, excludeList=None,verbose=False):
    '''
    @param cx: database connection
    @type cx: db API 2.0 object
    @param dbType: postgres or sqlite
    @type dbType: str
    @param includeList: If a list of message numbers is passed, only these are created
    @type includeList: list of integers
    @param excludeList: If a list of message numbers is passed, all but these are created
    @type excludeList: list of integers
    '''
    cu = cx.cursor()

    tables=[]
    for msgNum in ais.msgModByNumber:
        if excludeList is not None and msgNum in excludeList: continue
        if includeList is not None and msgNum not in includeList: continue

        aisMod = ais.msgModByNumber[msgNum]

        if aisMod.dbTableName in tables:
            if verbose: print msgNum,' ... skipping - already in the db -',aisMod.dbTableName
        else:
            if verbose:  print msgNum,' ... adding '+aisMod.dbTableName+' table to db'
            cu.execute(str(aisMod.sqlCreate(dbType=dbType)))
            tables.append(aisMod.dbTableName)

    cx.commit()

def dropTables(cx,dbType='sqlite',includeList=None, excludeList=None,verbose=False):
    '''
    Kiss your data goodbye

    @param cx: database connection
    @type cx: db API 2.0 object
    @param dbType: postgres or sqlite
    @type dbType: str
    @param includeList: If a list of message numbers is passed, only these are created
    @type includeList: list of integers
    @param excludeList: If a list of message numbers is passed, all but these are created
    @type excludeList: list of integers
    '''
    cu = cx.cursor()

    tables=[]
    for msgNum in ais.msgModByNumber:
        if excludeList is not None and msgNum in excludeList: continue
        if includeList is not None and msgNum not in includeList: continue

        aisMod = ais.msgModByNumber[msgNum]

        if aisMod.dbTableName in tables:
            if verbose: print msgNum,' ... skipping - already dropped from the db -',aisMod.dbTableName
        else:
            if verbose:  print msgNum,' ... dropping '+aisMod.dbTableName+' table to db'
            cu.execute('DROP TABLE '+aisMod.dbTableName+';')
            tables.append(aisMod.dbTableName)

    cx.commit()




def connect(options,dbType=None):
    '''
    options must include the above standard options
    '''
    verbose = options.verbose
    if dbType is None:
        dbType = options.dbType

    cx = None

    if dbType=='sqlite':
        import sqlite3
        cx = sqlite3.connect(options.databaseFilename)

    elif dbType=='postgres':
        import psycopg2 as psycopg
        # FIX: add password support
        connectStr = "dbname='"+options.databaseName+"' user='"+options.databaseUser+"' host='"+options.databaseHost+"'"

        if options.verbose:
            print 'Connect string:',connectStr
        cx = psycopg.connect(connectStr)

    else:
        sys.exit('Must specify a database type')
              
    return cx
