#!/usr/bin/env python
__version__ = '$Revision: 2275 $'.split()[1]
__date__ = '$Date: 2006-07-10 16:22:35 -0400 (Mon, 10 Jul 2006) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__ = '''
Connect to a socket and forward what is received to another port.
Filter to a list of AIS receivers/basestations.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: GPL v2
@since: Jan 2008
'''

import datetime
import unittest
import sys
import ais.sqlhelp # for sec2timestamp

class UscgNmea:
    def __init__(self,nmeaStr=None):
        '''
        Fields:
         - rssi ('s'): relative signal strength indicator
         - signalStrength ('d') - signal strendth in dBm
         - timeOfArrival ('T') - time of arrive from receiver - seconds within the minute
         - slotNumber ('S') - Receive slot number
         - station ('r' or 'b') - station name or id that received the message
         - stationTypeCode - first letter of the station name indicating 'b'asestation or 'r'eceive only (I think)
         - cg_sec - receive time of the message from the logging software.  Unix UTC second timestamp
         - timestamp - python datetime object in UTC derived from the cg_sec

        @todo: parse the other fields?

        @see: Maritime navigation and radiocommunication equipment and
              systems - Digital interfaces - Part 100: Single talker
              and multiple listeners - Extra requirements to IEC
              61162-1 for the UAIS. (80_330e_PAS) Draft...

        '''
        if None!=nmeaStr:
            #if len(nmeaStr)<6:
            #    # FIX: throw exception
            #    sys.stderr.write('Not a AIVDM... too short\n')
            
            fields = nmeaStr.split(',')
            self.cg_sec=float(fields[-1])
            self.timestamp = datetime.datetime.utcfromtimestamp(self.cg_sec)
            self.sqlTimestampStr = ais.sqlhelp.sec2timestamp(self.cg_sec)
            # See 80_330e_PAS
            # 
            self.nmeaType=fields[0][1:]
            self.totalSentences = int(fields[1])
            self.sentenceNum = int(fields[2])
            tmp = fields[3]
            if len(tmp)>0:
                self.sequentialMsgId = int(tmp)
            else:
                self.sequentialMsgId = None
            # FIX: make an int if the above is set
            self.aisChannel = fields[4] # 'A' or 'B'
            self.contents = fields[5]
            self.fillbits = int(fields[6].split('*')[0])
            self.checksumStr = fields[6].split('*')[1] # FIX: this is a hex string.  Convert?

            if self.sentenceNum==1:
                self.msgTypeChar=fields[5][0]
            else:
                self.msgTypeChar=None

            for i in range(len(fields)-1,5,-1):
                if len(fields[i])==0: 
                    continue # maybe it should throw a parse exception instead?
                f = fields[i]
                c = f[0] # first charater determines what the field is
		if c in ('b','r'):
		    self.station = f # FIX: think we want to keep the code in the first char
                    self.stationTypeCode = self.station[0]
                    continue
		    #break # Found it so ditch the for loop
                if c == 's':
                    self.rssi=int(f[1:])
                    continue
                if c == 'd':
                    self.signalStrength = int(f[1:])
                    continue
                if c == 'T':
                    self.timeOfArrival = float(f[1:])
                    continue
                if c == 'S':
                    self.slotNumber = int(f[1:])
                    continue
                
class TestUscgNmea(unittest.TestCase):
    def testUscgNmea(self):
        un = UscgNmea('!AIVDM,1,1,,B,15Cjtd0Oj;Jp7ilG7=UkKBoB0<06,0*63,s1234,d-119,T12.34567123,r003669958,S4321,1085889680')

        self.failUnlessEqual(un.nmeaType,'AIVDM')
        self.failUnlessEqual(un.totalSentences,1)
        self.failUnlessEqual(un.sentenceNum,1)
        self.failUnlessEqual(un.sequentialMsgId,None)
        self.failUnlessEqual(un.aisChannel,'B')
        self.failUnlessEqual(un.fillbits,0)
        self.failUnlessEqual(un.checksumStr,'63')

        self.failUnlessEqual(un.rssi,1234)
        self.failUnlessEqual(un.signalStrength,-119)
        self.failUnlessEqual(un.timeOfArrival,12.34567123)
        self.failUnlessEqual(un.slotNumber,4321)
        self.failUnlessEqual(un.station,'r003669958')
        self.failUnlessEqual(un.stationTypeCode,'r')
        self.failUnlessEqual(un.cg_sec,float(1085889680))
        print un.timestamp
        print un.sqlTimestampStr  # Hmmm... they look the same


############################################################
if __name__=='__main__':

    from optparse import OptionParser
    parser = OptionParser(usage="%prog [options]",
                          version="%prog "+__version__)

    parser.add_option('--doc-test',dest='doctest',default=False,action='store_true'
                      ,help='run the documentation tests')
    parser.add_option('--unit-test',dest='unittest',default=False,action='store_true'
                      ,help='run the unit tests')
    parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true'
                      ,help='Make the test output verbose')

    (options,args) = parser.parse_args()

    if options.doctest:
        success=True
        import os; print os.path.basename(sys.argv[0]), 'doctests ...',
        sys.argv= [sys.argv[0]]
        if options.verbose: sys.argv.append('-v')
        import doctest
        numfail,numtests=doctest.testmod()
        if numfail==0: print 'ok'
        else: 
            print 'FAILED'
            success=False

        if not success: sys.exit('Something Failed')
	#del success # Hide success from epydoc

    if options.unittest:
        sys.argv = [sys.argv[0]]
        if options.verbose: sys.argv.append('-v')
        unittest.main()
