#!/usr/bin/env python
__version__ = '$Revision: 8256 $'.split()[1]
__date__ = '$Date: 2008-01-15 16:48:24 -0500 (Tue, 15 Jan 2008) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Listen for connections.  When it receives data, shove the data into a postgis server.

Expects normalized AIS messages.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@status: under development
@license: GPL v2
@undocumented: __version__ __author__ __doc__ parser

@todo: make this work with any database
@todo: detect when one of the child threads dies from an exception/traceback
'''

#import shutil
import sys
#import os

import time
import socket
import thread
import select

import ais
import ais.sqlhelp
import aisutils.database
import aisutils.uscg
import psycopg2

######################################################################

def dateStr():
    '''
    String representing the day so that it sorts correctly
    @return: yyyy-mm-dd
    @rtype: str
    '''
    t=time.gmtime()
    d='%04d-%02d-%02d' % t[:3]
    return d

######################################################################

class DatabaseHandler:
    '''
    Queue handling for the database
    '''
    def __init__(self,connection,dbUpdateInterval=5., threshold=1, verbose=False, skipDB=False):
        #, maxLoops=5
        '''
        @param connection: database connection
        @param dbUpdateInterval: how many seconds between database updates
        @param threshold: How many messages in the queue before they go to the db?
        @param skipDB: Do not actually talk to the database
        '''
        #@param maxLoops: how many loops with data below the threshold before we commit the data?

        if not skipDB: 
            self.cx = connection
            self.cu = connection.cursor()
        else:
            self.cx = None
            self.cu = None
        import Queue
        self.q = Queue.Queue()
        self.running=True
        self.dbUpdateInterval = dbUpdateInterval
        self.threshold=threshold
        self.v = verbose
        self.stopped=False  # Set to true when this class is all done and committed
        self.skipDB=skipDB


    def stop(self):
        self.running=False
        if self.v: sys.stderr.write('Database handler stop scheduled\n')

    def commit(self):
        '''
        Slow loop that commits groups of position reports to the database
        @todo: Build the lines/shiptracks from the position reports for the ships with new positions.
        @todo: Where do I decimate reports received from multiple receivers?
        @todo: Need to track the ships in the put.
        '''
        q = self.q
        cx = self.cx
        cu = self.cu
        skipDB = self.skipDB

        vesselsSeen=set()

        limitPoints = 10 # max number of points per vessel

        size = q.qsize()
        if size >= self.threshold:
            # Don't try to flush incoming messages
            if self.v: sys.stderr.write('pulling from queue '+str(size)+'\n')
            for i in range(size):
                sqlStr,vessel=q.get()
                if vessel is not None:
                    vesselsSeen.add(vessel)
                if v: sys.stderr.write(str(i)+' '+sqlStr+'\n')
                if not skipDB: cu.execute(sqlStr)
            if self.v: sys.stderr.write('committing...\n')
            if not skipDB:
                cx.commit()
                if v:
                    sys.stderr.write('recalculate ship tracks vessels seen\n')
                    sys.stderr.write('  '+str(vesselsSeen)+'\n')
                if len(vesselsSeen)>0:
                    cu2 = self.cx.cursor()
                    vesselsUpdated=0
                    for vessel in vesselsSeen:
                        #print 'vessel',vessel
                        query='SELECT AsText(position) FROM position WHERE userid='+str(vessel)+' LIMIT '+str(limitPoints)+';'
                        #print query
                        cu2.execute(query)
                        linePoints=[]
                        lineLen=0
                        for row in cu2.fetchall():
                            lineLen+=1
                            #print 'row',row
                            
                            linePoints.append(row[0].split('(')[1].split(')')[0])
                        if lineLen<2:
                            sys.stderr.write('Line needs at least 2 points for vessel '+str(vessel)+'\n')
                            continue
                        #print 'LENGTH:',lineLen
                        #print 'points',linePoints
                        lineWKT='LINESTRING('+','.join(linePoints)+')'
                        #print 'wkt',lineWKT

                        #print 'FIX: do we replace or insert???'
                        # FIX: pull name from shipdata if we have it
                        # FIX: Make track_lines into a configuration option

                        cu2.execute('SELECT track_key FROM track_lines WHERE userid='+str(vessel)+'\n')
                        track_keys = cu2.fetchall()
                        print 'track_keys',track_keys
                        if len(track_keys)==0:
                            query = 'INSERT INTO track_lines (userid,track) VALUES (%s,GeomFromText(%s,4326));'
                            #print query
                            try:
                                cu.execute(query,(vessel,lineWKT))
                            except psycopg2.ProgrammingError,inst:
                               sys.stderr.write('psycopg2 execute flailed: '+str(inst)+'\n')
                            else:
                               vesselsUpdated += 1
                        elif len(track_keys)==1:
                            query = 'UPDATE track_lines SET track = GeomFromText(%s,4326) WHERE track_key = %s'
                            key = track_keys[0][0]
                            #print 'UPDATE...',query,lineWKT,key
                            try:
                                cu.execute(query,(lineWKT,key))
                            except psycopg2.ProgrammingError,inst:
                               sys.stderr.write('psycopg2 execute flailed: '+str(inst)+'\n')
                            else:
                                vesselsUpdated += 1
                        else:
                            sys.stderr.write('ERROR: database corrupted ... too many track lines for '+str(vessel)+'\n')
                            
                    if vesselsUpdated>0:
                        cx.commit()
                    sys.stderr.write('Updated tracks ... '+str(vesselsUpdated)+' tracks updated\n')
        
    def handler(self,unused=None):
        
        while self.running:
            time.sleep(self.dbUpdateInterval)
            self.commit()
        sys.stderr.write('Database handler stopping... begin final commit\n')
        self.commit()
        self.stopped=True
        
#    def put(self,sqlStr):
#        if self.running:
#            self.q.put(sqlStr)
                

class HandleAisConnection:
    def __init__(self,dataSocket,dbQueue,options,dbType='postgres'):
        '''
        @param dbQueue: Queue object to push SQL messages onto
        '''
        self.options = options
        sys.stderr.write('hac options in __init__ '+str(self.options)+'\n')
        self.dataSocket = dataSocket
        self.running=True
        self.dbQueue = dbQueue
        self.dbType = dbType
        try:
            self.timeout=options.timeout
        except:
            self.timeout=1.
        self.buf=None

    def handler(self,unused=None):
        sys.stderr.write('hac handler\n')
        v = self.options.verbose
        if v:
            sys.stderr.write('handler started.  VERBOSE\n')
        
        while self.running:
            #if v: sys.stderr.write('calling select\n')
            readersready,outputready,exceptready = select.select([self.dataSocket,],[],[],self.timeout)
            # FIX: how do we know that a socket is closed?
            #if v: sys.stderr.write('select returned\n')
            for sock in readersready:
                data = sock.recv(1024)
                #if v: sys.stderr.write('read '+str(len(data))+' bytes: '+data+'\n')

                if len(data)==0:
                    # Means we didn't time out and the socket is closed
                    # FIX: save the connection info and say who left
                    if v: sys.stderr.write('shutting down ais connection handler\n')
                    self.running=False
                    continue

                if self.buf is not None:
                    data=self.buf+data
                    if v: sys.stderr.write('new buf: '+data+'\n')
                    self.buf=None
                nmeaStrs=data.split('\n')
                if data[-1] not in ('\n','\r'):
                    #if v: sys.stderr.write('putting last entry in buffer.  last char is "'+data[-1]+'"\n')
                    self.buf=nmeaStrs.pop()

                if len(data)==0:
                    #if v: sys.stderr.write('nothing to process... partial message probably ion the buffer\n')
                    continue

                #if v: sys.stderr.write('processing '+str(len(nmeaStrs))+' message(s)\n')
                for msg in nmeaStrs:
                    if 'AIVDM'!= msg[1:6]:
                        if v:
                            sys.stderr.write('Skipping non ais message "'+msg+'"\n')
                        continue
                    else:
                        if v: sys.stderr.write('processing ais message ... '+msg+'\n')
                        pass

                    uscgMsg=aisutils.uscg.UscgNmea(msg)
                    #fields = msg.split(',')
                    #msgStr = fields[5]

                    if uscgMsg.totalSentences !=1:
                        if v:
                            sys.stderr.write('Can not handle unnormalized messages. Tot sentences: '+str(uscgMsg.totalSentences)+'\n')
                            sys.stderr.write('  msg:'+msg+'\n')
                        continue
                    
                    #msgTypeChar = msgStr[0]
                    if uscgMsg.msgTypeChar not in ais.msgModByFirstChar:
                        if v: sys.stderr.write('Can not process message starting with '+uscgMsg.msgTypeChar+'\n')
                        continue
                    aismsg = ais.msgModByFirstChar[uscgMsg.msgTypeChar]
                    #if v: sys.stderr.write('bits'+str(uscgMsg.contents)+'\n') #msgStr
                    bv = ais.binary.ais6tobitvec(uscgMsg.contents)
                    #if v: sys.stderr.write('bv '+str(bv)+'\n')
                    msgDict=aismsg.decode(bv)
                    ins = aismsg.sqlInsert(msgDict,dbType=self.dbType) #dbType='postgres')

                    cg_sec = uscgMsg.cg_sec
                    cg_timestamp = uscgMsg.sqlTimestampStr
                    cg_station = uscgMsg.station

                    if None != cg_sec:       ins.add('cg_sec',       cg_sec)
                    if None != cg_timestamp: ins.add('cg_timestamp', cg_timestamp)
                    if None != cg_station:   ins.add('cg_r',         cg_station)

                    #if v: sys.stderr.write('queuing ais SQL insert command: ' + str(ins)+'\n')

                    # Figure out if the vessel track needs to be updated
                    vessel=None
                    if aismsg.dbTableName in ('position',):
                        vessel=msgDict['UserID']
                    self.dbQueue.put((str(ins),vessel))

        
class PassThroughServer:
    '''Receive data from a socket and write the data to all clients that
    are connected.  Starts two threads and returns to the caller.
    '''
    nmeaInputs={} # indexed by socket... handles if we have partial text
    def __init__(self,options,dbHandler):
        '''
        @param options: understands timeout (float in seconds)
        '''
	self.clients=[]
	self.options = options
	self.count=0
        try:
            self.timeout=options.timeout
        except:
            self.timeout=1.
        try:
            self.hosts_allow=options.hosts_allow
        except:
            self.hosts_allow=None
        self.running=True
        self.hacs=[]
        self.dbHandler=dbHandler

    def start(self):
        '''
        start the thread
        FIX: cleanup and use verbose flag
        '''
	sys.stderr.write('starting threads\n')
	thread.start_new_thread(self.connection_handler,(self,))
        sys.stderr.write('connection_handler started\n')
	return

	
    def connection_handler(self,unused=None):
	'''Do not use this.  Call start() instead.  This listens for
	connections and adds the new socket to the clients list.

	@bug: how can I get rid of unused?
	'''
	sys.stderr.write('starting incoming connection receiver\n')

	serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
	serversocket.bind((self.options.inHost, self.options.inPort))
	serversocket.listen(5)

	while self.running:
	    (clientsocket, address) = serversocket.accept()
	    sys.stderr.write('connect from '+str(clientsocket)+' '+str(address)+'\n')
            if self.hosts_allow is not None and address[0] not in self.hosts_allow:
                sys.stderr.write('not allowing connection from ... '+str(address)+'\n')
                continue
            else:
                sys.stderr.write('creating hac\n')
                hac = HandleAisConnection(clientsocket
                                          ,self.dbHandler.q
                                          ,self.options
                                          ,self.options.dbType)
                sys.stderr.write('creating thread\n')
                thread.start_new_thread(hac.handler,(self,))
                self.hacs.append(hac)

        # FIX: do cleanup for graceful shutdown
        # Stop all the hacs
        # Close and flush the database handler

    def stop(self):
        self.running=False

######################################################################


if __name__=='__main__':
    from optparse import OptionParser

    # FIX: is importing __init__ safe?
    parser = OptionParser(usage="%prog [options]",
                            version="%prog "+__version__ + " ("+__date__+")")

    parser.add_option('-l','--log-file',dest='logFile',type='string',default=None,
			help='Write the stream through a log file [default: %default]')

    parser.add_option('-i','--in-port',dest='inPort',type='int', default=31402,
			help='Where the data comes from [default: %default]')
    parser.add_option('-I','--in-host',dest='inHost',type='string',default='localhost',
			help='What host to read data from [default: %default]')
    parser.add_option('--in-gethostname',dest='inHostname', action='store_true', default=False
			,help='Where the data comes from [default: %default]')

    parser.add_option('-a','--allow',action='append',dest='hosts_allow'
                         ,help='Add hosts to a list that are allowed to connect [default: all]')

    parser.add_option('-t','--timeout',dest='timeout',type='float', default='300',
                      help='Number of seconds to timeout after if no data [default: %default]')

    aisutils.database.stdCmdlineOptions(parser,'postgres')

    parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true'
                      ,help='Make the test output verbose')

    parser.add_option('--dummy-db',dest='skipDB',default=False,action='store_true'
                      ,help='Do not actually talk to database')

    (options,args) = parser.parse_args()

    v = options.verbose
    options.uscg=True # FIX: don't hack this in
    options.dbType = 'postgres' # FIX: don't force this

    cx = None
    if not options.skipDB:
        cx = aisutils.database.connect(options,dbType=options.dbType)

    dbHandler = DatabaseHandler(cx,verbose=v,skipDB=options.skipDB)
    
    if v: sys.stderr.write('hosts allowed: '+str(options.hosts_allow)+'\n')

    if options.inHostname:
	options.inHost=socket.gethostname()

    pts = PassThroughServer(options,dbHandler)
    pts.start()

    # Now start up the thread to send the messages to 
    thread.start_new_thread(dbHandler.handler,(None,))

    timeout = options.timeout
    del(options) # remove global to force self.options
    i = 0
    running = True

    try: 
        while running:
            i+=1
            time.sleep(timeout)
            if v: sys.stderr.write('ping '+str(i)+'\n')
    except KeyboardInterrupt:
        if v: sys.stderr.write('\bshutting down...\n')

    # FIX: nuke all the connections?

    dbHandler.stop()
    while not dbHandler.stopped:
        time.sleep(.1)
    if v: sys.stderr.write('Finished cleaning up ... goodbye\n')
    
