#!/usr/bin/env python
__version__ = '$Revision: 8256 $'.split()[1]
__date__ = '$Date: 2008-01-15 16:48:24 -0500 (Tue, 15 Jan 2008) $'.split()[1]
__author__ = 'Kurt Schwehr'

__doc__='''
Command line AIS database utilities.

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@copyright: 2008
@var __date__: Date of last svn commit
@status: under development
@license: GPL v2
@since: 2008
@undocumented: __version__ __author__ __doc__ parser
'''

#import os
import sys
import aisutils.database

############################################################
if __name__=='__main__':
    from optparse import OptionParser
    parser = OptionParser(usage="%prog [options]",version="%prog "+__version__)
    parser.add_option('-t','--database-type',dest='dbType',default=aisutils.database.dbTypes[0]
                      ,choices=aisutils.database.dbTypes
                      ,help='Which database type to use [default: %default]')
    parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true'
                      ,help='Make program output more verbose info as it runs')

    aisutils.database.stdCmdlineOptions(parser,'all')

    parser.add_option('-c','--create-database',dest='createDB',default=False, action='store_true'
                      ,help='Create the database.  Sqlite does not need this.')
    #
    # DB table creations
    #
    parser.add_option('-C','--create-tables',dest='createTables',default=False, action='store_true'
                      ,help='Create the tables in the database')

    parser.add_option('--drop-tables',dest='dropTables',default=False, action='store_true'
                      ,help='Remove the tables in the database.  DANGER - destroys data')

    parser.add_option('-i','--include-msg',action='append',dest='includeMsgs'
                      ,type='int',default=None
                      ,help='Make a list of messages to put in the db [default: %default]')
    parser.add_option('-e','--exclude-msg',action='append',dest='excludeMsgs'
                      ,type='int',default=None
                      ,help='Make a list of messages to put in the db [default: %default]')

    parser.add_option('--add-track-lines',action='store_true',dest='createTrackLinesTable'
                      ,default=False,help='Add a track lines table for postgis')

    parser.add_option('--drop-track-lines',action='store_true',dest='dropTrackLinesTable'
                      ,default=False,help='Add a track lines table for postgis')

    (options,args) = parser.parse_args()
    v = options.verbose
    if options.createDB and options.dbType!='sqlite':
        sys.exit('Have not yet implemented creating the database for non-sqlite dbs')

    cx = aisutils.database.connect(options)

    if options.createTables:
        if v: sys.stderr.write('Creating tables\n')
        aisutils.database.createTables(cx
                                       ,dbType=options.dbType
                                       ,includeList=options.includeMsgs
                                       ,excludeList=options.excludeMsgs
                                       ,verbose=v)

    if options.dropTables:
        if v: sys.stderr.write('Dropping tables.  Say goodbye to your data\n')
        aisutils.database.dropTables(cx
                                     ,dbType=options.dbType
                                     ,includeList=options.includeMsgs
                                     ,excludeList=options.excludeMsgs
                                     ,verbose=v)

    if options.createTrackLinesTable:
        sqlCreate = 'CREATE TABLE track_lines ( track_key SERIAL PRIMARY KEY, userid integer, name character varying(20));' 
        sqlCreate += " SELECT AddGeometryColumn('track_lines','track',4326,'LINESTRING',2);"
        if v: sys.stderr.write('Creating track_lines table\n')
        if v: sys.stderr.write(sqlCreate+'\n')
        cu = cx.cursor()
        cu.execute(sqlCreate)
        cx.commit()

    if options.dropTrackLinesTable:
        if v: sys.stderr.write('Dropping track_lines table ... \n')
        cu = cx.cursor()
        cu.execute('DROP TABLE track_lines;')
        cx.commit()

    if v: sys.stderr.write('Done.\n')
